@@ -27,14 +27,14 @@ extern "C" {
 // Based on Double Metropolis-Hastings
 // it is for Bayesian model averaging
 // ----------------------------------------------------------------------------|
-void gcgm_DMH_bdmcmc_ma( int *iter, int *burnin, int G[], int g_space[], double Ts[], double Ti[], double K[], int *p, 
+void gcgm_DMH_bdmcmc_ma( int *iter, int *burnin, int G[], int g_space[], double g_prior[], double Ts[], double Ti[], double K[], int *p, 
 			 double Z[], int R[], int *n, int *gcgm,
 			 double K_hat[], double p_links[],
 			 int *b, int *b_star, double D[], double Ds[], int *print )
 {
 	int print_c = *print, iteration = *iter, burn_in = *burnin, b1 = *b;
 	int index_selected_edge, selected_edge_i, selected_edge_j, selected_edge_ij, counter;
-	int ip, i, j, dim = *p, pxp = dim * dim, one = 1;
+	int ip, i, j, ij, dim = *p, pxp = dim * dim, one = 1;
 	int qp = dim * ( dim - 1 ) / 2;
 	double sum_weights = 0.0, sum_rates, weight_C;
 
@@ -86,6 +86,14 @@ void gcgm_DMH_bdmcmc_ma( int *iter, int *burnin, int G[], int g_space[], double
 			}
 	int sub_qp = counter;
 
+	vector<double> log_ratio_g_prior( pxp );	
+	for( j = 1; j < dim; j++ )
+		for( i = 0; i < j; i++ )
+		{
+			ij = j * dim + i;
+			log_ratio_g_prior[ij] = log( static_cast<double>( g_prior[ij] / ( 1 - g_prior[ij] ) ) );
+		}
+
 //-- Main loop for birth-death MCMC -------------------------------------------| 
 	GetRNGstate();
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc++ )
@@ -102,7 +110,7 @@ void gcgm_DMH_bdmcmc_ma( int *iter, int *burnin, int G[], int g_space[], double
 		// sampling from K and sigma for double Metropolis-Hastings
 		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 		
-		rates_bdmcmc_dmh_parallel( &rates[0], &G[0], &index_row[0], &index_col[0], &sub_qp, &Ds[0], &D[0], &sigma[0], &K[0], &sigma_dmh[0], &K_dmh[0], b, &dim );
+		rates_bdmcmc_dmh_parallel( &rates[0], &log_ratio_g_prior[0], &G[0], &index_row[0], &index_col[0], &sub_qp, &Ds[0], &D[0], &sigma[0], &K[0], &sigma_dmh[0], &K_dmh[0], b, &dim );
 		
 //----- saving result ---------------------------------------------------------|	
 		if( i_mcmc >= burn_in )
@@ -158,7 +166,7 @@ void gcgm_DMH_bdmcmc_ma( int *iter, int *burnin, int G[], int g_space[], double
 // Based on Double Metropolis-Hastings
 // it is for maximum a posterior probability estimation (MAP)
 // ----------------------------------------------------------------------------|
-void gcgm_DMH_bdmcmc_map( int *iter, int *burnin, int G[], int g_space[], double Ts[], double Ti[], double K[], int *p, 
+void gcgm_DMH_bdmcmc_map( int *iter, int *burnin, int G[], int g_space[], double g_prior[], double Ts[], double Ti[], double K[], int *p, 
 			 double Z[], int R[], int *n, int *gcgm,
 			 int all_graphs[], double all_weights[], double K_hat[], 
 			 char *sample_graphs[], double graph_weights[], int *size_sample_g,
@@ -166,7 +174,7 @@ void gcgm_DMH_bdmcmc_map( int *iter, int *burnin, int G[], int g_space[], double
 {
 	int print_c = *print, iteration = *iter, burn_in = *burnin, b1 = *b, count_all_g = 0;
 	int index_selected_edge, selected_edge_i, selected_edge_j, selected_edge_ij, counter, size_sample_graph = *size_sample_g;
-	int ip, i, j, dim = *p, pxp = dim * dim, one = 1;
+	int ip, i, j, ij, dim = *p, pxp = dim * dim, one = 1;
 	double sum_weights = 0.0, sum_rates, weight_C;
 	bool this_one;
 	int qp = dim * ( dim - 1 ) / 2;
@@ -222,6 +230,14 @@ void gcgm_DMH_bdmcmc_map( int *iter, int *burnin, int G[], int g_space[], double
 			}
 	int sub_qp = counter;
 
+	vector<double>log_ratio_g_prior( pxp );	
+	for( j = 1; j < dim; j++ )
+		for( i = 0; i < j; i++ )
+		{
+			ij = j * dim + i;
+			log_ratio_g_prior[ij] = log( static_cast<double>( g_prior[ij] / ( 1 - g_prior[ij] ) ) );
+		}
+
 //-- Main loop for birth-death MCMC -------------------------------------------| 
 	GetRNGstate();
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc++ )
@@ -238,7 +254,7 @@ void gcgm_DMH_bdmcmc_map( int *iter, int *burnin, int G[], int g_space[], double
 		// sampling from K and sigma for double Metropolis-Hastings
 		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 		
-		rates_bdmcmc_dmh_parallel( &rates[0], &G[0], &index_row[0], &index_col[0], &sub_qp, &Ds[0], &D[0], &sigma[0], &K[0], &sigma_dmh[0], &K_dmh[0], b, &dim );
+		rates_bdmcmc_dmh_parallel( &rates[0], &log_ratio_g_prior[0], &G[0], &index_row[0], &index_col[0], &sub_qp, &Ds[0], &D[0], &sigma[0], &K[0], &sigma_dmh[0], &K_dmh[0], b, &dim );
 
 //----- saving result ---------------------------------------------------------|	
 		if( i_mcmc >= burn_in )
@@ -318,14 +334,14 @@ void gcgm_DMH_bdmcmc_map( int *iter, int *burnin, int G[], int g_space[], double
 // Based on Double Metropolis-Hastings
 // it is for Bayesian model averaging
 // ----------------------------------------------------------------------------|
-void gcgm_DMH_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], int g_space[], double Ts[], double Ti[], double K[], int *p, 
+void gcgm_DMH_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], int g_space[], double g_prior[], double Ts[], double Ti[], double K[], int *p, 
 			 double Z[], int R[], int *n, int *gcgm,
 			 double K_hat[], double p_links[],
 			 int *b, int *b_star, double D[], double Ds[], int *multi_update, int *print )
 {
 	int print_c = *print, iteration = *iter, burn_in = *burnin, b1 = *b, multi_update_C = *multi_update;
 	int selected_edge_i, selected_edge_j, selected_edge_ij, counter;
-	int ip, i, j, dim = *p, pxp = dim * dim, one = 1;
+	int ip, i, j, ij, dim = *p, pxp = dim * dim, one = 1;
 	int qp = dim * ( dim - 1 ) / 2;
 	double sum_weights = 0.0, sum_rates, weight_C; 	
 
@@ -380,6 +396,14 @@ void gcgm_DMH_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], int g_spa
 			}
 	int sub_qp = counter;
 
+	vector<double> log_ratio_g_prior( pxp );	
+	for( j = 1; j < dim; j++ )
+		for( i = 0; i < j; i++ )
+		{
+			ij = j * dim + i;
+			log_ratio_g_prior[ij] = log( static_cast<double>( g_prior[ij] / ( 1 - g_prior[ij] ) ) );
+		}
+
 //-- Main loop for birth-death MCMC -------------------------------------------| 
 	GetRNGstate();
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc += size_index )
@@ -396,7 +420,7 @@ void gcgm_DMH_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], int g_spa
 		// sampling from K and sigma for double Metropolis-Hastings
 		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 		
-		rates_bdmcmc_dmh_parallel( &rates[0], &G[0], &index_row[0], &index_col[0], &sub_qp, &Ds[0], &D[0], &sigma[0], &K[0], &sigma_dmh[0], &K_dmh[0], b, &dim );
+		rates_bdmcmc_dmh_parallel( &rates[0], &log_ratio_g_prior[0], &G[0], &index_row[0], &index_col[0], &sub_qp, &Ds[0], &D[0], &sigma[0], &K[0], &sigma_dmh[0], &K_dmh[0], b, &dim );
 
 //----- saving result ---------------------------------------------------------|	
 		if( i_mcmc >= burn_in )
@@ -456,7 +480,7 @@ void gcgm_DMH_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], int g_spa
 // Based on Double Metropolis-Hastings
 // it is for maximum a posterior probability estimation (MAP)
 // ----------------------------------------------------------------------------|
-void gcgm_DMH_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], int g_space[], double Ts[], double Ti[], double K[], int *p, 
+void gcgm_DMH_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], int g_space[], double g_prior[], double Ts[], double Ti[], double K[], int *p, 
 			 double Z[], int R[], int *n, int *gcgm,
 			 int all_graphs[], double all_weights[], double K_hat[], 
 			 char *sample_graphs[], double graph_weights[], int *size_sample_g, int *counter_all_g,
@@ -465,7 +489,7 @@ void gcgm_DMH_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], int g_sp
 	int print_c = *print, multi_update_C = *multi_update, iteration = *iter, burn_in = *burnin, b1 = *b;
 	int count_all_g = *counter_all_g;
 	int selected_edge_i, selected_edge_j, selected_edge_ij, counter, size_sample_graph = *size_sample_g;
-	int ip, i, j, dim = *p, pxp = dim * dim, one = 1;
+	int ip, i, j, ij, dim = *p, pxp = dim * dim, one = 1;
 	int qp = dim * ( dim - 1 ) / 2;
 	double sum_weights = 0.0, sum_rates, weight_C; 
 	bool this_one;
@@ -524,6 +548,14 @@ void gcgm_DMH_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], int g_sp
 			}
 	int sub_qp = counter;
 
+	vector<double> log_ratio_g_prior( pxp );	
+	for( j = 1; j < dim; j++ )
+		for( i = 0; i < j; i++ )
+		{
+			ij = j * dim + i;
+			log_ratio_g_prior[ij] = log( static_cast<double>( g_prior[ij] / ( 1 - g_prior[ij] ) ) );
+		}
+
 //-- Main loop for birth-death MCMC -------------------------------------------| 
 	GetRNGstate();
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc += size_index )
@@ -540,7 +572,7 @@ void gcgm_DMH_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], int g_sp
 		// sampling from K and sigma for double Metropolis-Hastings
 		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 		
-		rates_bdmcmc_dmh_parallel( &rates[0], &G[0], &index_row[0], &index_col[0], &sub_qp, &Ds[0], &D[0], &sigma[0], &K[0], &sigma_dmh[0], &K_dmh[0], b, &dim );
+		rates_bdmcmc_dmh_parallel( &rates[0], &log_ratio_g_prior[0], &G[0], &index_row[0], &index_col[0], &sub_qp, &Ds[0], &D[0], &sigma[0], &K[0], &sigma_dmh[0], &K_dmh[0], b, &dim );
 
 //----- saving result ---------------------------------------------------------|	
 		if( i_mcmc >= burn_in )
@@ -624,7 +656,7 @@ void gcgm_DMH_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], int g_sp
 // Based on Double Metropolis-Hastings
 // it is for Bayesian model averaging
 // ----------------------------------------------------------------------------|
-void gcgm_DMH_rjmcmc_ma( int *iter, int *burnin, int G[], int g_space[], double Ts[], double Ti[], double K[], int *p, 
+void gcgm_DMH_rjmcmc_ma( int *iter, int *burnin, int G[], int g_space[], double g_prior[], double Ts[], double Ti[], double K[], int *p, 
 			 double Z[], int R[], int *n, int *gcgm,
 			 double K_hat[], int p_links[], 
 			 int *b, int *b_star, double D[], double Ds[], int *print )
@@ -679,6 +711,14 @@ void gcgm_DMH_rjmcmc_ma( int *iter, int *burnin, int G[], int g_space[], double
 		for( j = 0; j < dim; j++ ) size_node[i] += G[ip + j];
 	}
 	
+	vector<double> log_ratio_g_prior( pxp );	
+	for( j = 1; j < dim; j++ )
+		for( i = 0; i < j; i++ )
+		{
+			ij = j * dim + i;
+			log_ratio_g_prior[ij] = log( static_cast<double>( g_prior[ij] / ( 1 - g_prior[ij] ) ) );
+		}
+
 //-- Main loop for Reversible Jump MCMC ---------------------------------------| 
 	GetRNGstate();
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc++ )
@@ -692,9 +732,9 @@ void gcgm_DMH_rjmcmc_ma( int *iter, int *burnin, int G[], int g_space[], double
 		
 		// STEP 2: selecting edge and calculating alpha -----------------------| 
 		// Randomly selecting one edge: NOTE qp = p * ( p - 1 ) / 2 
-		randomEdge = static_cast<int>( runif( 0, 1 ) * qp );
+		randomEdge = static_cast<int>( unif_rand() * qp );
 
-		counter   = 0;
+		counter = 0;
 		for( j = 1; j < dim; j++ )
 			for( i = 0; i < j; i++ )
 			{
@@ -732,12 +772,12 @@ void gcgm_DMH_rjmcmc_ma( int *iter, int *burnin, int G[], int g_space[], double
 			   &dim, &p1, &p2, &jj,
 			   &Dijj, &Dij, &Djj );
 
-		alpha_ij = ( G[ij] ) ? ( logH_ij - logI_p ) : ( logI_p - logH_ij );
+		alpha_ij = ( G[ij] ) ? ( logH_ij - logI_p ) - log_ratio_g_prior[ij] : ( logI_p - logH_ij ) + log_ratio_g_prior[ij];
 
 		// -------- End calculating alpha -------------------------------------|
 		
 		// Selecting an edge and updating G (graph)
-		if( log( static_cast<double>( runif( 0, 1 ) ) ) < alpha_ij )
+		if( log( static_cast<double>( unif_rand() ) ) < alpha_ij )
 		{
 			G[ij] = 1 - G[ij];
 			G[selected_edge_i * dim + selected_edge_j] = G[ij];
@@ -775,7 +815,7 @@ void gcgm_DMH_rjmcmc_ma( int *iter, int *burnin, int G[], int g_space[], double
 // Based on Double Metropolis-Hastings
 // it is for maximum a posterior probability estimation (MAP)
 // ----------------------------------------------------------------------------|
-void gcgm_DMH_rjmcmc_map( int *iter, int *burnin, int G[], int g_space[], double Ts[], double Ti[], double K[], int *p, 
+void gcgm_DMH_rjmcmc_map( int *iter, int *burnin, int G[], int g_space[], double g_prior[], double Ts[], double Ti[], double K[], int *p, 
 			 double Z[], int R[], int *n, int *gcgm,
 			 int all_graphs[], double all_weights[], double K_hat[], 
 			 char *sample_graphs[], double graph_weights[], int *size_sample_g,
@@ -836,6 +876,14 @@ void gcgm_DMH_rjmcmc_map( int *iter, int *burnin, int G[], int g_space[], double
 		for( j = 0; j < dim; j++ ) size_node[i] += G[ip + j];
 	}
 
+	vector<double> log_ratio_g_prior( pxp );	
+	for( j = 1; j < dim; j++ )
+		for( i = 0; i < j; i++ )
+		{
+			ij = j * dim + i;
+			log_ratio_g_prior[ij] = log( static_cast<double>( g_prior[ij] / ( 1 - g_prior[ij] ) ) );
+		}
+
 //-- Main loop for Reversible Jump MCMC ---------------------------------------| 
 	GetRNGstate();
 	for( int i_mcmc = 0; i_mcmc < iteration; i_mcmc++ )
@@ -849,7 +897,7 @@ void gcgm_DMH_rjmcmc_map( int *iter, int *burnin, int G[], int g_space[], double
 		
 		// STEP 2: selecting edge and calculating alpha
 		// Randomly selecting one edge: NOTE qp = p * ( p - 1 ) / 2 
-		randomEdge = static_cast<int>( runif( 0, 1 ) * qp );
+		randomEdge = static_cast<int>( unif_rand() * qp );
 
 		counter   = 0;
 		for( j = 1; j < dim; j++ )
@@ -889,10 +937,11 @@ void gcgm_DMH_rjmcmc_map( int *iter, int *burnin, int G[], int g_space[], double
 			   &dim, &p1, &p2, &jj,
 			   &Dijj, &Dij, &Djj );
 
-		alpha_ij = ( G[ij] ) ? ( logH_ij - logI_p ) : ( logI_p - logH_ij );
+		//alpha_ij = ( G[ij] ) ? ( logH_ij - logI_p ) : ( logI_p - logH_ij );
+		alpha_ij = ( G[ij] ) ? ( logH_ij - logI_p ) - log_ratio_g_prior[ij] : ( logI_p - logH_ij ) + log_ratio_g_prior[ij];
 		// -------- End calculating alpha -------------------------------------|		
 		// Selecting an edge and updating G (graph)
-		if( log( static_cast<double>( runif( 0, 1 ) ) ) < alpha_ij )
+		if( log( static_cast<double>( unif_rand() ) ) < alpha_ij )
 		{
 			G[ij] = 1 - G[ij];
 			G[selected_edge_i * dim + selected_edge_j] = G[ij];
